""" Module to access the DataRetention endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from ...models import (
    ChannelListWithTeamData,
    DataRetentionPolicyCreate,
    DataRetentionPolicyWithTeamAndChannelCounts,
    DataRetentionPolicyWithTeamAndChannelIds,
    GetDataRetentionPoliciesCountResponse200,
    GlobalDataRetentionPolicy,
    RetentionPolicyForChannelList,
    RetentionPolicyForTeamList,
    SearchChannelsForRetentionPolicyJsonBody,
    SearchTeamsForRetentionPolicyJsonBody,
    StatusOK,
    Team,
)
from ..base import ApiBaseClass


class DataRetentionApi(ApiBaseClass):
    """Endpoint for getting data retention policy settings."""

    async def get_team_policies_for_user(
        self,
        user_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> RetentionPolicyForTeamList:
        """Get the policies which are applied to a user's teams

        Gets the policies which are applied to the all of the teams to which a
        user belongs.

        ##### License
        Requires an E20 license.

        Permissions:
            Must be logged in as the user or have the `manage_system`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `GetTeamPoliciesForUser <https://api.mattermost.com/#operation/GetTeamPoliciesForUser>`_
        """

        url = f"/users/{user_id}/data_retention/team_policies"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = RetentionPolicyForTeamList.parse_obj(response.json())

            return response200
        return response

    async def get_channel_policies_for_user(
        self,
        user_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> RetentionPolicyForChannelList:
        """Get the policies which are applied to a user's channels

        Gets the policies which are applied to the all of the channels to which
        a user belongs.

        ##### License
        Requires an E20 license.

        Permissions:
            Must be logged in as the user or have the `manage_system`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `GetChannelPoliciesForUser <https://api.mattermost.com/#operation/GetChannelPoliciesForUser>`_
        """

        url = f"/users/{user_id}/data_retention/channel_policies"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = RetentionPolicyForChannelList.parse_obj(response.json())

            return response200
        return response

    async def get_data_retention_policy(
        self,
    ) -> GlobalDataRetentionPolicy:
        """Get the global data retention policy

        Gets the current global data retention policy details from the server,
        including what data should be purged and the cutoff times for each data
        type that should be purged.

        ##### License
        Requires an E20 license.

        Permissions:
            Requires an active session but no other permissions.
        Minimum Server Version:
            4.3

        Api Reference:
            `GetDataRetentionPolicy <https://api.mattermost.com/#operation/GetDataRetentionPolicy>`_
        """

        url = "/data_retention/policy"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = GlobalDataRetentionPolicy.parse_obj(response.json())

            return response200
        return response

    async def get_data_retention_policies_count(
        self,
    ) -> GetDataRetentionPoliciesCountResponse200:
        """Get the number of granular data retention policies

        Gets the number of granular (i.e. team or channel-specific) data
        retention
        policies from the server.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_read_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `GetDataRetentionPoliciesCount <https://api.mattermost.com/#operation/GetDataRetentionPoliciesCount>`_
        """

        url = "/data_retention/policies_count"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = GetDataRetentionPoliciesCountResponse200.parse_obj(
                response.json()
            )

            return response200
        return response

    async def get_data_retention_policies(
        self,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[DataRetentionPolicyWithTeamAndChannelCounts]:
        """Get the granular data retention policies

        Gets details about the granular (i.e. team or channel-specific) data
        retention
        policies from the server.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_read_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `GetDataRetentionPolicies <https://api.mattermost.com/#operation/GetDataRetentionPolicies>`_
        """

        url = "/data_retention/policies"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = (
                    DataRetentionPolicyWithTeamAndChannelCounts.parse_obj(
                        response200_item_data
                    )
                )

                response200.append(response200_item)

            return response200
        return response

    async def create_data_retention_policy(
        self,
        *,
        json_body: Union[DataRetentionPolicyCreate, Dict],
    ) -> DataRetentionPolicyWithTeamAndChannelCounts:
        """Create a new granular data retention policy

        Creates a new granular data retention policy with the specified display
        name and post duration.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_write_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `CreateDataRetentionPolicy <https://api.mattermost.com/#operation/CreateDataRetentionPolicy>`_
        """

        url = "/data_retention/policies"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = DataRetentionPolicyWithTeamAndChannelCounts.parse_obj(
                response.json()
            )

            return response201
        return response

    async def get_data_retention_policy_by_id(
        self,
        policy_id: str,
    ) -> DataRetentionPolicyWithTeamAndChannelCounts:
        """Get a granular data retention policy

        Gets details about a granular data retention policies by ID.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_read_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `GetDataRetentionPolicyByID <https://api.mattermost.com/#operation/GetDataRetentionPolicyByID>`_
        """

        url = f"/data_retention/policies/{policy_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = DataRetentionPolicyWithTeamAndChannelCounts.parse_obj(
                response.json()
            )

            return response200
        return response

    async def delete_data_retention_policy(
        self,
        policy_id: str,
    ) -> StatusOK:
        """Delete a granular data retention policy

        Deletes a granular data retention policy.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_write_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `DeleteDataRetentionPolicy <https://api.mattermost.com/#operation/DeleteDataRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def patch_data_retention_policy(
        self,
        policy_id: str,
        *,
        json_body: Union[DataRetentionPolicyWithTeamAndChannelIds, Dict],
    ) -> DataRetentionPolicyWithTeamAndChannelCounts:
        """Patch a granular data retention policy

        Patches (i.e. replaces the fields of) a granular data retention policy.
        If any fields are omitted, they will not be changed.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_write_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `PatchDataRetentionPolicy <https://api.mattermost.com/#operation/PatchDataRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.patch(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = DataRetentionPolicyWithTeamAndChannelCounts.parse_obj(
                response.json()
            )

            return response200
        return response

    async def get_teams_for_retention_policy(
        self,
        policy_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[Team]:
        """Get the teams for a granular data retention policy

        Gets the teams to which a granular data retention policy is applied.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_read_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `GetTeamsForRetentionPolicy <https://api.mattermost.com/#operation/GetTeamsForRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}/teams"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Team.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def add_teams_to_retention_policy(
        self,
        policy_id: str,
        *,
        json_body: Union[List[str], Dict],
    ) -> StatusOK:
        """Add teams to a granular data retention policy

        Adds teams to a granular data retention policy.



        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_write_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `AddTeamsToRetentionPolicy <https://api.mattermost.com/#operation/AddTeamsToRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}/teams"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def remove_teams_from_retention_policy(
        self,
        policy_id: str,
        *,
        json_body: Union[List[str], Dict],
    ) -> StatusOK:
        """Delete teams from a granular data retention policy

        Delete teams from a granular data retention policy.



        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_write_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `RemoveTeamsFromRetentionPolicy <https://api.mattermost.com/#operation/RemoveTeamsFromRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}/teams"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def search_teams_for_retention_policy(
        self,
        policy_id: str,
        *,
        json_body: Union[SearchTeamsForRetentionPolicyJsonBody, Dict],
    ) -> List[Team]:
        """Search for the teams in a granular data retention policy

        Searches for the teams to which a granular data retention policy is
        applied.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_read_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `SearchTeamsForRetentionPolicy <https://api.mattermost.com/#operation/SearchTeamsForRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}/teams/search"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Team.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def get_channels_for_retention_policy(
        self,
        policy_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> ChannelListWithTeamData:
        """Get the channels for a granular data retention policy

        Gets the channels to which a granular data retention policy is applied.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_read_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `GetChannelsForRetentionPolicy <https://api.mattermost.com/#operation/GetChannelsForRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}/channels"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = ChannelListWithTeamData.parse_obj(response.json())

            return response200
        return response

    async def add_channels_to_retention_policy(
        self,
        policy_id: str,
        *,
        json_body: Union[List[str], Dict],
    ) -> StatusOK:
        """Add channels to a granular data retention policy

        Adds channels to a granular data retention policy.



        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_write_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `AddChannelsToRetentionPolicy <https://api.mattermost.com/#operation/AddChannelsToRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}/channels"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def remove_channels_from_retention_policy(
        self,
        policy_id: str,
        *,
        json_body: Union[List[str], Dict],
    ) -> StatusOK:
        """Delete channels from a granular data retention policy

        Delete channels from a granular data retention policy.



        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_write_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `RemoveChannelsFromRetentionPolicy <https://api.mattermost.com/#operation/RemoveChannelsFromRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}/channels"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def search_channels_for_retention_policy(
        self,
        policy_id: str,
        *,
        json_body: Union[SearchChannelsForRetentionPolicyJsonBody, Dict],
    ) -> ChannelListWithTeamData:
        """Search for the channels in a granular data retention policy

        Searches for the channels to which a granular data retention policy is
        applied.

        ##### License
        Requires an E20 license.

        Permissions:
            Must have the `sysconsole_read_compliance_data_retention`
            permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `SearchChannelsForRetentionPolicy <https://api.mattermost.com/#operation/SearchChannelsForRetentionPolicy>`_
        """

        url = f"/data_retention/policies/{policy_id}/channels/search"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = ChannelListWithTeamData.parse_obj(response.json())

            return response200
        return response



""" Module to access the Search endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Dict, Union

from ...models import FileInfoList, SearchFilesMultipartData
from ..base import ApiBaseClass


class SearchApi(ApiBaseClass):
    """ """

    def search_files(
        self,
        team_id: str,
        *,
        multipart_data: Union[SearchFilesMultipartData, Dict],
    ) -> FileInfoList:
        """Search files in a team

        Search for files in a team based on file name, extention and file
        content (if file content extraction is enabled and supported for the
        files).

        Permissions:
            Must be authenticated and have the `view_team` permission.
        Minimum Server Version:
            5.34

        Api Reference:
            `SearchFiles <https://api.mattermost.com/#operation/SearchFiles>`_
        """

        url = f"/teams/{team_id}/files/search"

        multipart_body_data = SearchFilesMultipartData.parse_obj(multipart_data)

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = FileInfoList.parse_obj(response.json())

            return response200
        return response



""" Module to access the Files endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, Optional, Union

from ...models import (
    FileInfo,
    FileInfoList,
    GetFileLinkResponse200,
    SearchFilesMultipartData,
    UploadFileMultipartData,
    UploadFileResponse201,
)
from ..base import ApiBaseClass


class FilesApi(ApiBaseClass):
    """Endpoints for uploading and interacting with files."""

    async def upload_file(
        self,
        *,
        multipart_data: Union[UploadFileMultipartData, Dict],
        channel_id: Optional[str] = None,
        filename: Optional[str] = None,
    ) -> UploadFileResponse201:
        """Upload a file

        Uploads a file that can later be attached to a post.

        This request can either be a multipart/form-data request with a
        channel_id, files and optional
        client_ids defined in the FormData, or it can be a request with the
        channel_id and filename
        defined as query parameters with the contents of a single file in the
        body of the request.

        Only multipart/form-data requests are supported by server versions up to
        and including 4.7.
        Server versions 4.8 and higher support both types of requests.

        Permissions:
            Must have `upload_file` permission.

        Api Reference:
            `UploadFile <https://api.mattermost.com/#operation/UploadFile>`_
        """

        url = "/files"
        params: Dict[str, Any] = {
            "channel_id": channel_id,
            "filename": filename,
        }
        params = {k: v for k, v in params.items() if v is not None}

        multipart_body_data = UploadFileMultipartData.parse_obj(multipart_data)

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = UploadFileResponse201.parse_obj(response.json())

            return response201
        return response

    async def get_file(
        self,
        file_id: str,
    ) -> None:
        """Get a file

        Gets a file that has been uploaded previously.

        Permissions:
            Must have `read_channel` permission or be uploader of the
            file.

        Api Reference:
            `GetFile <https://api.mattermost.com/#operation/GetFile>`_
        """

        url = f"/files/{file_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def get_file_thumbnail(
        self,
        file_id: str,
    ) -> None:
        """Get a file's thumbnail

        Gets a file's thumbnail.

        Permissions:
            Must have `read_channel` permission or be uploader of the
            file.

        Api Reference:
            `GetFileThumbnail <https://api.mattermost.com/#operation/GetFileThumbnail>`_
        """

        url = f"/files/{file_id}/thumbnail"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def get_file_preview(
        self,
        file_id: str,
    ) -> None:
        """Get a file's preview

        Gets a file's preview.

        Permissions:
            Must have `read_channel` permission or be uploader of the
            file.

        Api Reference:
            `GetFilePreview <https://api.mattermost.com/#operation/GetFilePreview>`_
        """

        url = f"/files/{file_id}/preview"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def get_file_link(
        self,
        file_id: str,
    ) -> GetFileLinkResponse200:
        """Get a public file link

        Gets a public link for a file that can be accessed without logging into
        Mattermost.

        Permissions:
            Must have `read_channel` permission or be uploader of the
            file.

        Api Reference:
            `GetFileLink <https://api.mattermost.com/#operation/GetFileLink>`_
        """

        url = f"/files/{file_id}/link"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = GetFileLinkResponse200.parse_obj(response.json())

            return response200
        return response

    async def get_file_info(
        self,
        file_id: str,
    ) -> FileInfo:
        """Get metadata for a file

        Gets a file's info.

        Permissions:
            Must have `read_channel` permission or be uploader of the
            file.

        Api Reference:
            `GetFileInfo <https://api.mattermost.com/#operation/GetFileInfo>`_
        """

        url = f"/files/{file_id}/info"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = FileInfo.parse_obj(response.json())

            return response200
        return response

    async def get_file_public(
        self,
        file_id: str,
        *,
        h: str,
    ) -> None:
        """Get a public file



        Permissions:
            No permissions required.

        Api Reference:
            `GetFilePublic <https://api.mattermost.com/#operation/GetFilePublic>`_
        """

        url = f"/files/{file_id}/public"
        params: Dict[str, Any] = {
            "h": h,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def search_files(
        self,
        team_id: str,
        *,
        multipart_data: Union[SearchFilesMultipartData, Dict],
    ) -> FileInfoList:
        """Search files in a team

        Search for files in a team based on file name, extention and file
        content (if file content extraction is enabled and supported for the
        files).

        Permissions:
            Must be authenticated and have the `view_team` permission.
        Minimum Server Version:
            5.34

        Api Reference:
            `SearchFiles <https://api.mattermost.com/#operation/SearchFiles>`_
        """

        url = f"/teams/{team_id}/files/search"

        multipart_body_data = SearchFilesMultipartData.parse_obj(multipart_data)

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = FileInfoList.parse_obj(response.json())

            return response200
        return response



""" Module to access the System endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union, cast

from pydantic import BaseModel

from ...models import (
    Audit,
    Config,
    EnvironmentConfig,
    IntegrityCheckResult,
    LicenseRenewalLink,
    Notice,
    PostLogJsonBody,
    PostLogResponse200,
    PushNotification,
    RequestTrialLicenseJsonBody,
    SendWarnMetricAckJsonBody,
    Server_Busy,
    StatusOK,
    System,
    SystemStatusResponse,
    TestSiteURLJsonBody,
    UpgradeToEnterpriseStatusResponse200,
    UploadLicenseFileMultipartData,
)
from ..base import ApiBaseClass


class SystemApi(ApiBaseClass):
    """General endpoints for interacting with the server, such as configuration
    and logging."""

    async def get_supported_timezone(
        self,
    ) -> List[str]:
        """Retrieve a list of supported timezones



        Permissions:
            Must be logged in.
        Minimum Server Version:
            3.10

        Api Reference:
            `GetSupportedTimezone <https://api.mattermost.com/#operation/GetSupportedTimezone>`_
        """

        url = "/system/timezones"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = cast(List[str], response.json())

            return response200
        return response

    async def get_ping(
        self,
        *,
        get_server_status: Optional[bool] = None,
        device_id: Optional[str] = None,
    ) -> SystemStatusResponse:
        """Check system health

        Check if the server is up and healthy based on the configuration setting
        `GoRoutineHealthThreshold`. If `GoRoutineHealthThreshold` and the number
        of goroutines on the server exceeds that threshold the server is
        considered unhealthy. If `GoRoutineHealthThreshold` is not set or the
        number of goroutines is below the threshold the server is considered
        healthy.
        If a \"device_id\" is passed in the query, it will test the Push
        Notification Proxy in order to discover whether the device is able to
        receive notifications. The response will have a
        \"CanReceiveNotifications\" property with one of the following values: -
        true: It can receive notifications - false: It cannot receive
        notifications - unknown: There has been an unknown error, and it is not
        certain whether it can
          receive notifications.

        Permissions:
            None.
        Minimum Server Version:
            3.10

        Api Reference:
            `GetPing <https://api.mattermost.com/#operation/GetPing>`_
        """

        url = "/system/ping"
        params: Dict[str, Any] = {
            "get_server_status": get_server_status,
            "device_id": device_id,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = SystemStatusResponse.parse_obj(response.json())

            return response200
        return response

    async def get_notices(
        self,
        teamId: str,
        *,
        clientVersion: str,
        locale: Optional[str] = None,
        client: str,
    ) -> List[Notice]:
        """Get notices for logged in user in specified team

        Will return appropriate product notices for current user in the team
        specified by teamId parameter.

        Permissions:
            Must be logged in.
        Minimum Server Version:
            5.26

        Api Reference:
            `GetNotices <https://api.mattermost.com/#operation/GetNotices>`_
        """

        url = f"/system/notices/{teamId}"
        params: Dict[str, Any] = {
            "clientVersion": clientVersion,
            "locale": locale,
            "client": client,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Notice.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def mark_notices_viewed(
        self,
        *,
        json_body: Union[List[str], Dict],
    ) -> StatusOK:
        """Update notices as 'viewed'

        Will mark the specified notices as 'viewed' by the logged in user.

        Permissions:
            Must be logged in.
        Minimum Server Version:
            5.26

        Api Reference:
            `MarkNoticesViewed <https://api.mattermost.com/#operation/MarkNoticesViewed>`_
        """

        url = "/system/notices/view"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def database_recycle(
        self,
    ) -> StatusOK:
        """Recycle database connections

        Recycle database connections by closing and reconnecting all connections
        to master and read replica databases.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `DatabaseRecycle <https://api.mattermost.com/#operation/DatabaseRecycle>`_
        """

        url = "/database/recycle"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def test_email(
        self,
        *,
        json_body: Union[Config, Dict],
    ) -> StatusOK:
        """Send a test email

        Send a test email to make sure you have your email settings configured
        correctly. Optionally provide a configuration in the request body to
        test. If no valid configuration is present in the request body the
        current server configuration will be tested.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `TestEmail <https://api.mattermost.com/#operation/TestEmail>`_
        """

        url = "/email/test"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def test_site_url(
        self,
        *,
        json_body: Union[TestSiteURLJsonBody, Dict],
    ) -> StatusOK:
        """Checks the validity of a Site URL

        Sends a Ping request to the mattermost server using the specified Site
        URL.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.16

        Api Reference:
            `TestSiteURL <https://api.mattermost.com/#operation/TestSiteURL>`_
        """

        url = "/site_url/test"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def test_s3_connection(
        self,
        *,
        json_body: Union[Config, Dict],
    ) -> StatusOK:
        """Test AWS S3 connection

        Send a test to validate if can connect to AWS S3. Optionally provide a
        configuration in the request body to test. If no valid configuration is
        present in the request body the current server configuration will be
        tested.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.8

        Api Reference:
            `TestS3Connection <https://api.mattermost.com/#operation/TestS3Connection>`_
        """

        url = "/file/s3_test"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def get_config(
        self,
    ) -> Config:
        """Get configuration

        Retrieve the current server configuration

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `GetConfig <https://api.mattermost.com/#operation/GetConfig>`_
        """

        url = "/config"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Config.parse_obj(response.json())

            return response200
        return response

    async def update_config(
        self,
        *,
        json_body: Union[Config, Dict],
    ) -> Config:
        """Update configuration

        Submit a new configuration for the server to use. As of server version
        4.8, the `PluginSettings.EnableUploads` setting cannot be modified by
        this endpoint.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `UpdateConfig <https://api.mattermost.com/#operation/UpdateConfig>`_
        """

        url = "/config"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Config.parse_obj(response.json())

            return response200
        return response

    async def reload_config(
        self,
    ) -> StatusOK:
        """Reload configuration

        Reload the configuration file to pick up on any changes made to it.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `ReloadConfig <https://api.mattermost.com/#operation/ReloadConfig>`_
        """

        url = "/config/reload"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def get_client_config(
        self,
        *,
        format: str,
    ) -> None:
        """Get client configuration

        Get a subset of the server configuration needed by the client.

        Permissions:
            No permission required.

        Api Reference:
            `GetClientConfig <https://api.mattermost.com/#operation/GetClientConfig>`_
        """

        url = "/config/client"
        params: Dict[str, Any] = {
            "format": format,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def get_environment_config(
        self,
    ) -> EnvironmentConfig:
        """Get configuration made through environment variables

        Retrieve a json object mirroring the server configuration where fields
        are set to true
        if the corresponding config setting is set through an environment
        variable. Settings
        that haven't been set through environment variables will be missing from
        the object.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.10

        Api Reference:
            `GetEnvironmentConfig <https://api.mattermost.com/#operation/GetEnvironmentConfig>`_
        """

        url = "/config/environment"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = EnvironmentConfig.parse_obj(response.json())

            return response200
        return response

    async def patch_config(
        self,
        *,
        json_body: Union[Config, Dict],
    ) -> Config:
        """Patch configuration

        Submit configuration to patch. As of server version 4.8, the
        `PluginSettings.EnableUploads` setting cannot be modified by this
        endpoint.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.20

        Api Reference:
            `PatchConfig <https://api.mattermost.com/#operation/PatchConfig>`_
        """

        url = "/config/patch"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Config.parse_obj(response.json())

            return response200
        return response

    async def upload_license_file(
        self,
        *,
        multipart_data: Union[UploadLicenseFileMultipartData, Dict],
    ) -> StatusOK:
        """Upload license file

        Upload a license to enable enterprise features.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.0

        Api Reference:
            `UploadLicenseFile <https://api.mattermost.com/#operation/UploadLicenseFile>`_
        """

        url = "/license"

        multipart_body_data = UploadLicenseFileMultipartData.parse_obj(multipart_data)

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = StatusOK.parse_obj(response.json())

            return response201
        return response

    async def remove_license_file(
        self,
    ) -> None:
        """Remove license file

        Remove the license file from the server. This will disable all
        enterprise features.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.0

        Api Reference:
            `RemoveLicenseFile <https://api.mattermost.com/#operation/RemoveLicenseFile>`_
        """

        url = "/license"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def get_client_license(
        self,
        *,
        format: str,
    ) -> None:
        """Get client license

        Get a subset of the server license needed by the client.

        Permissions:
            No permission required but having the `manage_system`
            permission returns more information.

        Api Reference:
            `GetClientLicense <https://api.mattermost.com/#operation/GetClientLicense>`_
        """

        url = "/license/client"
        params: Dict[str, Any] = {
            "format": format,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def request_license_renewal_link(
        self,
    ) -> LicenseRenewalLink:
        """Request the license renewal link

        Request the renewal link that would be used to start the license renewal
        process

        Permissions:
            Must have `sysconsole_write_about` permission.
        Minimum Server Version:
            5.32

        Api Reference:
            `RequestLicenseRenewalLink <https://api.mattermost.com/#operation/RequestLicenseRenewalLink>`_
        """

        url = "/license/renewal"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = LicenseRenewalLink.parse_obj(response.json())

            return response200
        return response

    async def request_trial_license(
        self,
        *,
        json_body: Union[RequestTrialLicenseJsonBody, Dict],
    ) -> None:
        """Request and install a trial license for your server

        Request and install a trial license for your server

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.25

        Api Reference:
            `RequestTrialLicense <https://api.mattermost.com/#operation/RequestTrialLicense>`_
        """

        url = "/trial-license"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def get_prev_trial_license(
        self,
    ) -> None:
        """Get last trial license used

        Get the last trial license used on the sevrer

        Permissions:
            Must have `manage_systems` permissions.
        Minimum Server Version:
            5.36

        Api Reference:
            `GetPrevTrialLicense <https://api.mattermost.com/#operation/GetPrevTrialLicense>`_
        """

        url = "/trial-license/prev"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def get_audits(
        self,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[Audit]:
        """Get audits

        Get a page of audits for all users on the system, selected with `page`
        and `per_page` query parameters.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `GetAudits <https://api.mattermost.com/#operation/GetAudits>`_
        """

        url = "/audits"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Audit.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def invalidate_caches(
        self,
    ) -> StatusOK:
        """Invalidate all the caches

        Purge all the in-memory caches for the Mattermost server. This can have
        a temporary negative effect on performance while the caches are re-
        populated.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `InvalidateCaches <https://api.mattermost.com/#operation/InvalidateCaches>`_
        """

        url = "/caches/invalidate"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def get_logs(
        self,
        *,
        page: Optional[int] = 0,
        logs_per_page: Optional[str] = "10000",
    ) -> List[str]:
        """Get logs

        Get a page of server logs, selected with `page` and `logs_per_page`
        query parameters.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `GetLogs <https://api.mattermost.com/#operation/GetLogs>`_
        """

        url = "/logs"
        params: Dict[str, Any] = {
            "page": page,
            "logs_per_page": logs_per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = cast(List[str], response.json())

            return response200
        return response

    async def post_log(
        self,
        *,
        json_body: Union[PostLogJsonBody, Dict],
    ) -> PostLogResponse200:
        """Add log message

        Add log messages to the server logs.
        Logged in users can log ERROR or DEBUG messages when
        `ServiceSettings.EnableDeveloper` is `true` or just DEBUG messages when
        `false`.
        Non-logged in users can log ERROR or DEBUG messages when
        `ServiceSettings.EnableDeveloper` is `true` and cannot log when `false`.

        Permissions:
            Users with `manage_system` permission can log ERROR or DEBUG
            messages.

        Api Reference:
            `PostLog <https://api.mattermost.com/#operation/PostLog>`_
        """

        url = "/logs"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = PostLogResponse200.parse_obj(response.json())

            return response200
        return response

    async def get_analytics_old(
        self,
        *,
        name: Optional[str] = "standard",
        team_id: Optional[str] = None,
    ) -> None:
        """Get analytics

        Get some analytics data about the system. This endpoint uses the old
        format, the `/analytics` route is reserved for the new format when it
        gets implemented.

        The returned JSON changes based on the `name` query parameter but is
        always key/value pairs.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.0

        Api Reference:
            `GetAnalyticsOld <https://api.mattermost.com/#operation/GetAnalyticsOld>`_
        """

        url = "/analytics/old"
        params: Dict[str, Any] = {
            "name": name,
            "team_id": team_id,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def get_server_busy_expires(
        self,
    ) -> Server_Busy:
        """Get server busy expiry time.

        Gets the timestamp corresponding to when the server busy flag will be
        automatically cleared.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.20

        Api Reference:
            `GetServerBusyExpires <https://api.mattermost.com/#operation/GetServerBusyExpires>`_
        """

        url = "/server_busy"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Server_Busy.parse_obj(response.json())

            return response200
        return response

    async def set_server_busy(
        self,
        *,
        seconds: Optional[str] = "3600",
    ) -> StatusOK:
        """Set the server busy (high load) flag

        Marks the server as currently having high load which disables non-
        critical services such as search, statuses and typing notifications.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.20

        Api Reference:
            `SetServerBusy <https://api.mattermost.com/#operation/SetServerBusy>`_
        """

        url = "/server_busy"
        params: Dict[str, Any] = {
            "seconds": seconds,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def clear_server_busy(
        self,
    ) -> StatusOK:
        """Clears the server busy (high load) flag

        Marks the server as not having high load which re-enables non-critical
        services such as search, statuses and typing notifications.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.20

        Api Reference:
            `ClearServerBusy <https://api.mattermost.com/#operation/ClearServerBusy>`_
        """

        url = "/server_busy"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def get_redirect_location(
        self,
        *,
        url: str,
    ) -> None:
        """Get redirect location



        Permissions:
            Must be logged in.
        Minimum Server Version:
            3.10

        Api Reference:
            `GetRedirectLocation <https://api.mattermost.com/#operation/GetRedirectLocation>`_
        """

        url = "/redirect_location"
        params: Dict[str, Any] = {
            "url": url,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def get_image_by_url(
        self,
    ) -> None:
        """Get an image by url

        Fetches an image via Mattermost image proxy.

        Permissions:
            Must be logged in.
        Minimum Server Version:
            3.10

        Api Reference:
            `GetImageByUrl <https://api.mattermost.com/#operation/GetImageByUrl>`_
        """

        url = "/image"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def upgrade_to_enterprise(
        self,
    ) -> PushNotification:
        """Executes an inplace upgrade from Team Edition to Enterprise Edition

        It downloads the Mattermost Enterprise Edition of your current version
        and replace your current version with it. After the upgrade you need to
        restart the Mattermost server.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.27

        Api Reference:
            `UpgradeToEnterprise <https://api.mattermost.com/#operation/UpgradeToEnterprise>`_
        """

        url = "/upgrade_to_enterprise"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 202:
            response202 = PushNotification.parse_obj(response.json())

            return response202
        return response

    async def upgrade_to_enterprise_status(
        self,
    ) -> UpgradeToEnterpriseStatusResponse200:
        """Get the current status for the inplace upgrade from Team Edition to
        Enterprise Edition

        It returns the percentage of completion of the current upgrade or the
        error if there is any.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.27

        Api Reference:
            `UpgradeToEnterpriseStatus <https://api.mattermost.com/#operation/UpgradeToEnterpriseStatus>`_
        """

        url = "/upgrade_to_enterprise/status"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = UpgradeToEnterpriseStatusResponse200.parse_obj(
                response.json()
            )

            return response200
        return response

    async def restart_server(
        self,
    ) -> StatusOK:
        """Restart the system after an upgrade from Team Edition to Enterprise
        Edition

        It restarts the current running mattermost instance to execute the new
        Enterprise binary.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.27

        Api Reference:
            `RestartServer <https://api.mattermost.com/#operation/RestartServer>`_
        """

        url = "/restart"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def get_warn_metrics_status(
        self,
    ) -> StatusOK:
        """Get the warn metrics status (enabled or disabled)

        Get the status of a set of metrics (enabled or disabled) from the
        Systems table.

        The returned JSON contains the metrics that we need to warn the admin on
        with regard
        to their status (we return the ones whose status is \"true\", which
        means that they are
        in a \"warnable\" state - e.g. a threshold has been crossed or some
        other condition has
        been fulfilled).

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `GetWarnMetricsStatus <https://api.mattermost.com/#operation/GetWarnMetricsStatus>`_
        """

        url = "/warn_metrics/status"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def send_warn_metric_ack(
        self,
        warn_metric_id: str,
        *,
        json_body: Union[SendWarnMetricAckJsonBody, Dict],
    ) -> StatusOK:
        """Acknowledge a warning of a metric status

        Acknowledge a warning for the warn_metric_id metric crossing a threshold
        (or some
        similar condition being fulfilled) - attempts to send an ack email to
        acknowledge@mattermost.com and sets the \"ack\" status for all the warn
        metrics in the system.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `SendWarnMetricAck <https://api.mattermost.com/#operation/SendWarnMetricAck>`_
        """

        url = f"/warn_metrics/ack/{warn_metric_id}"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def send_trial_license_warn_metric_ack(
        self,
        warn_metric_id: str,
    ) -> StatusOK:
        """Request trial license and acknowledge a warning of a metric status

        Request a trial license and acknowledge a warning for the warn_metric_id
        metric crossing a threshold (or some
        similar condition being fulfilled) - sets the \"ack\" status for all the
        warn metrics in the system.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.28

        Api Reference:
            `SendTrialLicenseWarnMetricAck <https://api.mattermost.com/#operation/SendTrialLicenseWarnMetricAck>`_
        """

        url = f"/warn_metrics/trial-license-ack/{warn_metric_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def check_integrity(
        self,
    ) -> List[IntegrityCheckResult]:
        """Perform a database integrity check

        Performs a database integrity check.

        Minimum Server Version:
            5.28.0
        Local Mode Only:
            This endpoint is only available through [local
            mode](https://docs.mattermost.com/administration/mmctl-cli-
            tool.html#local-mode).
        Warning:
            This check may temporarily harm system performance.

        Api Reference:
            `CheckIntegrity <https://api.mattermost.com/#operation/CheckIntegrity>`_
        """

        url = "/integrity"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = IntegrityCheckResult.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def generate_support_packet(
        self,
    ) -> None:
        """Download a zip file which contains helpful and useful information for
        troubleshooting your mattermost instance.

        Download a zip file which contains helpful and useful information for
        troubleshooting your mattermost instance.
        ##### License
        Requires either a E10 or E20 license.

        Permissions:
            Must have any of the system console read permissions.
        Minimum Server Version:
            5.32

        Api Reference:
            `GenerateSupportPacket <https://api.mattermost.com/#operation/GenerateSupportPacket>`_
        """

        url = "/system/support_packet"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def update_marketplace_visited_by_admin(
        self,
        *,
        json_body: Union[System, Dict],
    ) -> StatusOK:
        """Stores that the Plugin Marketplace has been visited by at least an
        admin.

        Stores the system-level status that specifies that at least an admin has
        visited the in-product Plugin Marketplace.

        Permissions:
            Must have `manage_system` permissions.
        Minimum Server Version:
            5.33

        Api Reference:
            `UpdateMarketplaceVisitedByAdmin <https://api.mattermost.com/#operation/UpdateMarketplaceVisitedByAdmin>`_
        """

        url = "/plugins/marketplace/first_admin_visit"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response



""" Enums used in different API endpoints """
from enum import Enum


class PluginStatusState(str, Enum):
    """State of the plugin"""

    NOTRUNNING = "NotRunning"
    STARTING = "Starting"
    RUNNING = "Running"
    FAILEDTOSTART = "FailedToStart"
    FAILEDTOSTAYRUNNING = "FailedToStayRunning"
    STOPPING = "Stopping"

    def __str__(self) -> str:
        return str(self.value)


class SidebarCategoryType(str, Enum):
    """None"""

    CHANNELS = "channels"
    CUSTOM = "custom"
    DIRECT_MESSAGES = "direct_messages"
    FAVORITES = "favorites"

    def __str__(self) -> str:
        return str(self.value)


class SidebarCategoryWithChannelsType(str, Enum):
    """None"""

    CHANNELS = "channels"
    CUSTOM = "custom"
    DIRECT_MESSAGES = "direct_messages"
    FAVORITES = "favorites"

    def __str__(self) -> str:
        return str(self.value)


class UploadSessionType(str, Enum):
    """The type of the upload."""

    ATTACHMENT = "attachment"
    IMPORT_ = "import"

    def __str__(self) -> str:
        return str(self.value)


class PostMetadataEmbedsItemType(str, Enum):
    """The type of content that is embedded in this point."""

    IMAGE = "image"
    MESSAGE_ATTACHMENT = "message_attachment"
    OPENGRAPH = "opengraph"
    LINK = "link"

    def __str__(self) -> str:
        return str(self.value)



""" Module to access the Commands endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from ...models import (
    AutocompleteSuggestion,
    Command,
    CommandResponse,
    CreateCommandJsonBody,
    ExecuteCommandJsonBody,
    MoveCommandJsonBody,
    RegenCommandTokenResponse200,
    StatusOK,
)
from ..base import ApiBaseClass


class CommandsApi(ApiBaseClass):
    """Endpoints for creating, getting and updating slash commands."""

    async def list_commands(
        self,
        *,
        team_id: Optional[str] = None,
        custom_only: Optional[bool] = False,
    ) -> List[Command]:
        """List commands for a team

        List commands for a team.

        Permissions:
            `manage_slash_commands` if need list custom commands.

        Api Reference:
            `ListCommands <https://api.mattermost.com/#operation/ListCommands>`_
        """

        url = "/commands"
        params: Dict[str, Any] = {
            "team_id": team_id,
            "custom_only": custom_only,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Command.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def create_command(
        self,
        *,
        json_body: Union[CreateCommandJsonBody, Dict],
    ) -> Command:
        """Create a command

        Create a command for a team.

        Permissions:
            `manage_slash_commands` for the team the command is in.

        Api Reference:
            `CreateCommand <https://api.mattermost.com/#operation/CreateCommand>`_
        """

        url = "/commands"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Command.parse_obj(response.json())

            return response201
        return response

    async def list_autocomplete_commands(
        self,
        team_id: str,
    ) -> List[Command]:
        """List autocomplete commands

        List autocomplete commands in the team.

        Permissions:
            `view_team` for the team.

        Api Reference:
            `ListAutocompleteCommands <https://api.mattermost.com/#operation/ListAutocompleteCommands>`_
        """

        url = f"/teams/{team_id}/commands/autocomplete"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Command.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def list_command_autocomplete_suggestions(
        self,
        team_id: str,
        *,
        user_input: str,
    ) -> List[AutocompleteSuggestion]:
        """List commands' autocomplete data

        List commands' autocomplete data for the team.

        Permissions:
            `view_team` for the team.
        Minimum Server Version:
            5.24

        Api Reference:
            `ListCommandAutocompleteSuggestions <https://api.mattermost.com/#operation/ListCommandAutocompleteSuggestions>`_
        """

        url = f"/teams/{team_id}/commands/autocomplete_suggestions"
        params: Dict[str, Any] = {
            "user_input": user_input,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = AutocompleteSuggestion.parse_obj(
                    response200_item_data
                )

                response200.append(response200_item)

            return response200
        return response

    async def get_command_by_id(
        self,
        command_id: str,
    ) -> Command:
        """Get a command

        Get a command definition based on command id string.

        Permissions:
            Must have `manage_slash_commands` permission for the team
            the command is in.
        Minimum Server Version:
            5.22

        Api Reference:
            `GetCommandById <https://api.mattermost.com/#operation/GetCommandById>`_
        """

        url = f"/commands/{command_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Command.parse_obj(response.json())

            return response200
        return response

    async def update_command(
        self,
        command_id: str,
        *,
        json_body: Union[Command, Dict],
    ) -> Command:
        """Update a command

        Update a single command based on command id string and Command struct.

        Permissions:
            Must have `manage_slash_commands` permission for the team
            the command is in.

        Api Reference:
            `UpdateCommand <https://api.mattermost.com/#operation/UpdateCommand>`_
        """

        url = f"/commands/{command_id}"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Command.parse_obj(response.json())

            return response200
        return response

    async def delete_command(
        self,
        command_id: str,
    ) -> StatusOK:
        """Delete a command

        Delete a command based on command id string.

        Permissions:
            Must have `manage_slash_commands` permission for the team
            the command is in.

        Api Reference:
            `DeleteCommand <https://api.mattermost.com/#operation/DeleteCommand>`_
        """

        url = f"/commands/{command_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def move_command(
        self,
        command_id: str,
        *,
        json_body: Union[MoveCommandJsonBody, Dict],
    ) -> StatusOK:
        """Move a command

        Move a command to a different team based on command id string.

        Permissions:
            Must have `manage_slash_commands` permission for the team
            the command is currently in and the destination team.
        Minimum Server Version:
            5.22

        Api Reference:
            `MoveCommand <https://api.mattermost.com/#operation/MoveCommand>`_
        """

        url = f"/commands/{command_id}/move"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def regen_command_token(
        self,
        command_id: str,
    ) -> RegenCommandTokenResponse200:
        """Generate a new token

        Generate a new token for the command based on command id string.

        Permissions:
            Must have `manage_slash_commands` permission for the team
            the command is in.

        Api Reference:
            `RegenCommandToken <https://api.mattermost.com/#operation/RegenCommandToken>`_
        """

        url = f"/commands/{command_id}/regen_token"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = RegenCommandTokenResponse200.parse_obj(response.json())

            return response200
        return response

    async def execute_command(
        self,
        *,
        json_body: Union[ExecuteCommandJsonBody, Dict],
    ) -> CommandResponse:
        """Execute a command

        Execute a command on a team.

        Permissions:
            Must have `use_slash_commands` permission for the team the
            command is in.

        Api Reference:
            `ExecuteCommand <https://api.mattermost.com/#operation/ExecuteCommand>`_
        """

        url = "/commands/execute"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = CommandResponse.parse_obj(response.json())

            return response200
        return response



""" Async client to access the mattermost API """

from collections.abc import AsyncGenerator
from contextlib import asynccontextmanager
from typing import Optional

import httpx
from pydantic import PrivateAttr

from ..endpoints.async_api.authentication import AuthenticationApi
from ..endpoints.async_api.bleve import BleveApi
from ..endpoints.async_api.bots import BotsApi
from ..endpoints.async_api.brand import BrandApi
from ..endpoints.async_api.channels import ChannelsApi
from ..endpoints.async_api.cloud import CloudApi
from ..endpoints.async_api.cluster import ClusterApi
from ..endpoints.async_api.commands import CommandsApi
from ..endpoints.async_api.compliance import ComplianceApi
from ..endpoints.async_api.data_retention import DataRetentionApi
from ..endpoints.async_api.elasticsearch import ElasticsearchApi
from ..endpoints.async_api.emoji import EmojiApi
from ..endpoints.async_api.exports import ExportsApi
from ..endpoints.async_api.files import FilesApi
from ..endpoints.async_api.groups import GroupsApi
from ..endpoints.async_api.imports import ImportsApi
from ..endpoints.async_api.integration_actions import IntegrationActionsApi
from ..endpoints.async_api.jobs import JobsApi
from ..endpoints.async_api.ldap import LdapApi
from ..endpoints.async_api.migrate import MigrateApi
from ..endpoints.async_api.oauth import OauthApi
from ..endpoints.async_api.open_graph import OpenGraphApi
from ..endpoints.async_api.permissions import PermissionsApi
from ..endpoints.async_api.plugins import PluginsApi
from ..endpoints.async_api.posts import PostsApi
from ..endpoints.async_api.preferences import PreferencesApi
from ..endpoints.async_api.reactions import ReactionsApi
from ..endpoints.async_api.roles import RolesApi
from ..endpoints.async_api.root import RootApi
from ..endpoints.async_api.saml import SamlApi
from ..endpoints.async_api.schemes import SchemesApi
from ..endpoints.async_api.search import SearchApi
from ..endpoints.async_api.shared_channels import SharedChannelsApi
from ..endpoints.async_api.status import StatusApi
from ..endpoints.async_api.system import SystemApi
from ..endpoints.async_api.teams import TeamsApi
from ..endpoints.async_api.terms_of_service import TermsOfServiceApi
from ..endpoints.async_api.threads import ThreadsApi
from ..endpoints.async_api.uploads import UploadsApi
from ..endpoints.async_api.users import UsersApi
from ..endpoints.async_api.webhooks import WebhooksApi
from .base import AuthLogin, AuthToken, BaseClient, HttpxClientOptions
from .exceptions import (
    ContentTooLarge,
    FeatureDisabled,
    InternalServerError,
    InvalidOrMissingParameters,
    MethodNotAllowed,
    NoAccessTokenProvided,
    NotEnoughPermissions,
    ResourceNotFound,
    TooManyRequests,
)


async def raise_on_4xx_5xx(response):
    try:
        response.raise_for_status()
    except httpx.HTTPStatusError as http_error:
        try:
            await http_error.response.aread()
            data = http_error.response.json()
            message = data.get("message", data)
        except ValueError:
            message = response.text
        if http_error.response.status_code == 400:
            raise InvalidOrMissingParameters(message) from http_error
        if http_error.response.status_code == 401:
            raise NoAccessTokenProvided(message) from http_error
        if http_error.response.status_code == 403:
            raise NotEnoughPermissions(message) from http_error
        if http_error.response.status_code == 404:
            raise ResourceNotFound(message) from http_error
        if http_error.response.status_code == 405:
            raise MethodNotAllowed(message) from http_error
        if http_error.response.status_code == 413:
            raise ContentTooLarge(message) from http_error
        if http_error.response.status_code == 429:
            raise TooManyRequests(message) from http_error
        if http_error.response.status_code == 500:
            raise InternalServerError(message) from http_error
        if http_error.response.status_code == 501:
            raise FeatureDisabled(message) from http_error
        raise


class AsyncClient(BaseClient):
    """Synchronous mattermost api client implementation"""

    _httpx_client: Optional[httpx.AsyncClient] = PrivateAttr(None)
    """ The underlying httpx client which handles requests to the api in case we are inside a session """

    async def _create_httpx_client(self):
        """Create a httpx.AsyncClient instance to be used for requests and perform authentication if needed"""
        httpx_client_options = (
            self.options.httpx_client_options
            if self.options.httpx_client_options
            else HttpxClientOptions()
        )
        base_url = str(httpx.URL(self.options.url).join(self.options.basepath))
        httpx_client = httpx.AsyncClient(
            base_url=base_url, **dict(httpx_client_options)
        )
        httpx_client.event_hooks["response"] = [
            raise_on_4xx_5xx
        ] + httpx_client.event_hooks["response"]
        if isinstance(self.options.auth, AuthToken):
            httpx_client.auth = self.options.auth
            self.active_token = self.options.auth.token
        if isinstance(self.options.auth, AuthLogin):
            # Login with username and password and get a session_token
            response = await httpx_client.post(
                url="/users/login", json=self.options.auth.dict(exclude_unset=True)
            )
            session_token = response.headers.get("token", None)
            httpx_client.auth = AuthToken(token=session_token)
            self.active_token = session_token
        return httpx_client

    async def _login(self):
        """Calling this creates a httpx client and sets .active_token, needed for websockets"""
        await self._create_httpx_client()

    @asynccontextmanager
    async def _get_httpx_client(self):
        """Get the currently set httpx.Client instance or create a new one"""
        if not self._httpx_client or self._httpx_client.is_closed:
            httpx_client = await self._create_httpx_client()
            try:
                yield httpx_client
            finally:
                await httpx_client.aclose()
        else:
            yield self._httpx_client

    @asynccontextmanager
    async def session(self) -> AsyncGenerator["AsyncClient", None, None]:
        """Open a Session which re-uses the underlying httpx client and it's connections"""
        # pylint: disable=protected-access
        api_client = self.copy()
        api_client._httpx_client = await api_client._create_httpx_client()
        try:
            yield api_client
        finally:
            await api_client._httpx_client.aclose()

    @property
    def users(self) -> UsersApi:
        """Api endpoint for Users

        :type: :class:`~matterapi.endpoints.async_api.UsersApi`
        """
        return UsersApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def bots(self) -> BotsApi:
        """Api endpoint for Bots

        :type: :class:`~matterapi.endpoints.async_api.BotsApi`
        """
        return BotsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def terms_of_service(self) -> TermsOfServiceApi:
        """Api endpoint for TermsOfService

        :type: :class:`~matterapi.endpoints.async_api.TermsOfServiceApi`
        """
        return TermsOfServiceApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def migrate(self) -> MigrateApi:
        """Api endpoint for Migrate

        :type: :class:`~matterapi.endpoints.async_api.MigrateApi`
        """
        return MigrateApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def authentication(self) -> AuthenticationApi:
        """Api endpoint for Authentication

        :type: :class:`~matterapi.endpoints.async_api.AuthenticationApi`
        """
        return AuthenticationApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def ldap(self) -> LdapApi:
        """Api endpoint for Ldap

        :type: :class:`~matterapi.endpoints.async_api.LdapApi`
        """
        return LdapApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def saml(self) -> SamlApi:
        """Api endpoint for Saml

        :type: :class:`~matterapi.endpoints.async_api.SamlApi`
        """
        return SamlApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def threads(self) -> ThreadsApi:
        """Api endpoint for Threads

        :type: :class:`~matterapi.endpoints.async_api.ThreadsApi`
        """
        return ThreadsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def data_retention(self) -> DataRetentionApi:
        """Api endpoint for DataRetention

        :type: :class:`~matterapi.endpoints.async_api.DataRetentionApi`
        """
        return DataRetentionApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def status(self) -> StatusApi:
        """Api endpoint for Status

        :type: :class:`~matterapi.endpoints.async_api.StatusApi`
        """
        return StatusApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def teams(self) -> TeamsApi:
        """Api endpoint for Teams

        :type: :class:`~matterapi.endpoints.async_api.TeamsApi`
        """
        return TeamsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def channels(self) -> ChannelsApi:
        """Api endpoint for Channels

        :type: :class:`~matterapi.endpoints.async_api.ChannelsApi`
        """
        return ChannelsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def posts(self) -> PostsApi:
        """Api endpoint for Posts

        :type: :class:`~matterapi.endpoints.async_api.PostsApi`
        """
        return PostsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def preferences(self) -> PreferencesApi:
        """Api endpoint for Preferences

        :type: :class:`~matterapi.endpoints.async_api.PreferencesApi`
        """
        return PreferencesApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def files(self) -> FilesApi:
        """Api endpoint for Files

        :type: :class:`~matterapi.endpoints.async_api.FilesApi`
        """
        return FilesApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def search(self) -> SearchApi:
        """Api endpoint for Search

        :type: :class:`~matterapi.endpoints.async_api.SearchApi`
        """
        return SearchApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def uploads(self) -> UploadsApi:
        """Api endpoint for Uploads

        :type: :class:`~matterapi.endpoints.async_api.UploadsApi`
        """
        return UploadsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def jobs(self) -> JobsApi:
        """Api endpoint for Jobs

        :type: :class:`~matterapi.endpoints.async_api.JobsApi`
        """
        return JobsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def system(self) -> SystemApi:
        """Api endpoint for System

        :type: :class:`~matterapi.endpoints.async_api.SystemApi`
        """
        return SystemApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def root(self) -> RootApi:
        """Api endpoint for Root

        :type: :class:`~matterapi.endpoints.async_api.RootApi`
        """
        return RootApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def emoji(self) -> EmojiApi:
        """Api endpoint for Emoji

        :type: :class:`~matterapi.endpoints.async_api.EmojiApi`
        """
        return EmojiApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def webhooks(self) -> WebhooksApi:
        """Api endpoint for Webhooks

        :type: :class:`~matterapi.endpoints.async_api.WebhooksApi`
        """
        return WebhooksApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def compliance(self) -> ComplianceApi:
        """Api endpoint for Compliance

        :type: :class:`~matterapi.endpoints.async_api.ComplianceApi`
        """
        return ComplianceApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def groups(self) -> GroupsApi:
        """Api endpoint for Groups

        :type: :class:`~matterapi.endpoints.async_api.GroupsApi`
        """
        return GroupsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def cluster(self) -> ClusterApi:
        """Api endpoint for Cluster

        :type: :class:`~matterapi.endpoints.async_api.ClusterApi`
        """
        return ClusterApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def brand(self) -> BrandApi:
        """Api endpoint for Brand

        :type: :class:`~matterapi.endpoints.async_api.BrandApi`
        """
        return BrandApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def commands(self) -> CommandsApi:
        """Api endpoint for Commands

        :type: :class:`~matterapi.endpoints.async_api.CommandsApi`
        """
        return CommandsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def oauth(self) -> OauthApi:
        """Api endpoint for Oauth

        :type: :class:`~matterapi.endpoints.async_api.OauthApi`
        """
        return OauthApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def elasticsearch(self) -> ElasticsearchApi:
        """Api endpoint for Elasticsearch

        :type: :class:`~matterapi.endpoints.async_api.ElasticsearchApi`
        """
        return ElasticsearchApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def bleve(self) -> BleveApi:
        """Api endpoint for Bleve

        :type: :class:`~matterapi.endpoints.async_api.BleveApi`
        """
        return BleveApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def plugins(self) -> PluginsApi:
        """Api endpoint for Plugins

        :type: :class:`~matterapi.endpoints.async_api.PluginsApi`
        """
        return PluginsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def roles(self) -> RolesApi:
        """Api endpoint for Roles

        :type: :class:`~matterapi.endpoints.async_api.RolesApi`
        """
        return RolesApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def schemes(self) -> SchemesApi:
        """Api endpoint for Schemes

        :type: :class:`~matterapi.endpoints.async_api.SchemesApi`
        """
        return SchemesApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def shared_channels(self) -> SharedChannelsApi:
        """Api endpoint for SharedChannels

        :type: :class:`~matterapi.endpoints.async_api.SharedChannelsApi`
        """
        return SharedChannelsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def open_graph(self) -> OpenGraphApi:
        """Api endpoint for OpenGraph

        :type: :class:`~matterapi.endpoints.async_api.OpenGraphApi`
        """
        return OpenGraphApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def reactions(self) -> ReactionsApi:
        """Api endpoint for Reactions

        :type: :class:`~matterapi.endpoints.async_api.ReactionsApi`
        """
        return ReactionsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def integration_actions(self) -> IntegrationActionsApi:
        """Api endpoint for IntegrationActions

        :type: :class:`~matterapi.endpoints.async_api.IntegrationActionsApi`
        """
        return IntegrationActionsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def cloud(self) -> CloudApi:
        """Api endpoint for Cloud

        :type: :class:`~matterapi.endpoints.async_api.CloudApi`
        """
        return CloudApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def permissions(self) -> PermissionsApi:
        """Api endpoint for Permissions

        :type: :class:`~matterapi.endpoints.async_api.PermissionsApi`
        """
        return PermissionsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def imports(self) -> ImportsApi:
        """Api endpoint for Imports

        :type: :class:`~matterapi.endpoints.async_api.ImportsApi`
        """
        return ImportsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )

    @property
    def exports(self) -> ExportsApi:
        """Api endpoint for Exports

        :type: :class:`~matterapi.endpoints.async_api.ExportsApi`
        """
        return ExportsApi(
            client=self, skip_response_parsing=self.options.skip_response_parsing
        )



""" Module to access the TermsOfService endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Dict, Union

from pydantic import BaseModel

from ...models import (
    AppError,
    RegisterTermsOfServiceActionJsonBody,
    StatusOK,
    TermsOfService,
    UserTermsOfService,
)
from ..base import ApiBaseClass


class TermsOfServiceApi(ApiBaseClass):
    """Endpoints for getting and updating custom terms of service."""

    def get_user_terms_of_service(
        self,
        user_id: str,
    ) -> Union[AppError, UserTermsOfService]:
        """Fetches user's latest terms of service action if the latest action was
        for acceptance.

        Will be deprecated in v6.0
        Fetches user's latest terms of service action if the latest action was
        for acceptance.

        Permissions:
            Must be logged in as the user being acted on.
        Minimum Server Version:
            5.6

        Api Reference:
            `GetUserTermsOfService <https://api.mattermost.com/#operation/GetUserTermsOfService>`_
        """

        url = f"/users/{user_id}/terms_of_service"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = UserTermsOfService.parse_obj(response.json())

            return response200
        if response.status_code == 404:
            response404 = AppError.parse_obj(response.json())

            return response404
        return response

    def register_terms_of_service_action(
        self,
        user_id: str,
        *,
        json_body: Union[RegisterTermsOfServiceActionJsonBody, Dict],
    ) -> StatusOK:
        """Records user action when they accept or decline custom terms of service

        Records user action when they accept or decline custom terms of service.
        Records the action in audit table.
        Updates user's last accepted terms of service ID if they accepted it.

        Permissions:
            Must be logged in as the user being acted on.
        Minimum Server Version:
            5.4

        Api Reference:
            `RegisterTermsOfServiceAction <https://api.mattermost.com/#operation/RegisterTermsOfServiceAction>`_
        """

        url = f"/users/{user_id}/terms_of_service"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def get_terms_of_service(
        self,
    ) -> TermsOfService:
        """Get latest terms of service

        Get latest terms of service from the server

        Permissions:
            Must be authenticated.
        Minimum Server Version:
            5.4

        Api Reference:
            `GetTermsOfService <https://api.mattermost.com/#operation/GetTermsOfService>`_
        """

        url = "/terms_of_service"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = TermsOfService.parse_obj(response.json())

            return response200
        return response

    def create_terms_of_service(
        self,
    ) -> TermsOfService:
        """Creates a new terms of service

        Creates new terms of service

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.4

        Api Reference:
            `CreateTermsOfService <https://api.mattermost.com/#operation/CreateTermsOfService>`_
        """

        url = "/terms_of_service"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = TermsOfService.parse_obj(response.json())

            return response200
        return response



""" Module to access the Bots endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from ...models import (
    Bot,
    ConvertBotToUserJsonBody,
    CreateBotJsonBody,
    PatchBotJsonBody,
    SetBotIconImageMultipartData,
    StatusOK,
)
from ..base import ApiBaseClass


class BotsApi(ApiBaseClass):
    """Endpoints for creating, getting and updating bot users."""

    def convert_user_to_bot(
        self,
        user_id: str,
    ) -> StatusOK:
        """Convert a user into a bot

        Convert a user into a bot.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `ConvertUserToBot <https://api.mattermost.com/#operation/ConvertUserToBot>`_
        """

        url = f"/users/{user_id}/convert_to_bot"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def get_bots(
        self,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
        include_deleted: Optional[bool] = None,
        only_orphaned: Optional[bool] = None,
    ) -> List[Bot]:
        """Get bots

        Get a page of a list of bots.

        Permissions:
            Must have `read_bots` permission for bots you are managing,
            and `read_others_bots` permission for bots others are
            managing.
        Minimum Server Version:
            5.10

        Api Reference:
            `GetBots <https://api.mattermost.com/#operation/GetBots>`_
        """

        url = "/bots"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
            "include_deleted": include_deleted,
            "only_orphaned": only_orphaned,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Bot.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def create_bot(
        self,
        *,
        json_body: Union[CreateBotJsonBody, Dict],
    ) -> Bot:
        """Create a bot

        Create a new bot account on the system. Username is required.

        Permissions:
            Must have `create_bot` permission.
        Minimum Server Version:
            5.10

        Api Reference:
            `CreateBot <https://api.mattermost.com/#operation/CreateBot>`_
        """

        url = "/bots"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Bot.parse_obj(response.json())

            return response201
        return response

    def get_bot(
        self,
        bot_user_id: str,
        *,
        include_deleted: Optional[bool] = None,
    ) -> Bot:
        """Get a bot

        Get a bot specified by its bot id.

        Permissions:
            Must have `read_bots` permission for bots you are managing,
            and `read_others_bots` permission for bots others are
            managing.
        Minimum Server Version:
            5.10

        Api Reference:
            `GetBot <https://api.mattermost.com/#operation/GetBot>`_
        """

        url = f"/bots/{bot_user_id}"
        params: Dict[str, Any] = {
            "include_deleted": include_deleted,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Bot.parse_obj(response.json())

            return response200
        return response

    def patch_bot(
        self,
        bot_user_id: str,
        *,
        json_body: Union[PatchBotJsonBody, Dict],
    ) -> Bot:
        """Patch a bot

        Partially update a bot by providing only the fields you want to update.
        Omitted fields will not be updated. The fields that can be updated are
        defined in the request body, all other provided fields will be ignored.

        Permissions:
            Must have `manage_bots` permission.
        Minimum Server Version:
            5.10

        Api Reference:
            `PatchBot <https://api.mattermost.com/#operation/PatchBot>`_
        """

        url = f"/bots/{bot_user_id}"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Bot.parse_obj(response.json())

            return response200
        return response

    def disable_bot(
        self,
        bot_user_id: str,
    ) -> Bot:
        """Disable a bot

        Disable a bot.

        Permissions:
            Must have `manage_bots` permission.
        Minimum Server Version:
            5.10

        Api Reference:
            `DisableBot <https://api.mattermost.com/#operation/DisableBot>`_
        """

        url = f"/bots/{bot_user_id}/disable"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Bot.parse_obj(response.json())

            return response200
        return response

    def enable_bot(
        self,
        bot_user_id: str,
    ) -> Bot:
        """Enable a bot

        Enable a bot.

        Permissions:
            Must have `manage_bots` permission.
        Minimum Server Version:
            5.10

        Api Reference:
            `EnableBot <https://api.mattermost.com/#operation/EnableBot>`_
        """

        url = f"/bots/{bot_user_id}/enable"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Bot.parse_obj(response.json())

            return response200
        return response

    def assign_bot(
        self,
        bot_user_id: str,
        user_id: str,
    ) -> Bot:
        """Assign a bot to a user

        Assign a bot to a specified user.

        Permissions:
            Must have `manage_bots` permission.
        Minimum Server Version:
            5.10

        Api Reference:
            `AssignBot <https://api.mattermost.com/#operation/AssignBot>`_
        """

        url = f"/bots/{bot_user_id}/assign/{user_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Bot.parse_obj(response.json())

            return response200
        return response

    def get_bot_icon_image(
        self,
        bot_user_id: str,
    ) -> None:
        """Get bot's LHS icon

        Get a bot's LHS icon image based on bot_user_id string parameter.

        Permissions:
            Must be logged in.
        Minimum Server Version:
            5.14

        Api Reference:
            `GetBotIconImage <https://api.mattermost.com/#operation/GetBotIconImage>`_
        """

        url = f"/bots/{bot_user_id}/icon"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def set_bot_icon_image(
        self,
        bot_user_id: str,
        *,
        multipart_data: Union[SetBotIconImageMultipartData, Dict],
    ) -> StatusOK:
        """Set bot's LHS icon image

        Set a bot's LHS icon image based on bot_user_id string parameter. Icon
        image must be SVG format, all other formats are rejected.

        Permissions:
            Must have `manage_bots` permission.
        Minimum Server Version:
            5.14

        Api Reference:
            `SetBotIconImage <https://api.mattermost.com/#operation/SetBotIconImage>`_
        """

        url = f"/bots/{bot_user_id}/icon"

        multipart_body_data = SetBotIconImageMultipartData.parse_obj(multipart_data)

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def delete_bot_icon_image(
        self,
        bot_user_id: str,
    ) -> StatusOK:
        """Delete bot's LHS icon image

        Delete bot's LHS icon image based on bot_user_id string parameter.

        Permissions:
            Must have `manage_bots` permission.
        Minimum Server Version:
            5.14

        Api Reference:
            `DeleteBotIconImage <https://api.mattermost.com/#operation/DeleteBotIconImage>`_
        """

        url = f"/bots/{bot_user_id}/icon"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def convert_bot_to_user(
        self,
        bot_user_id: str,
        *,
        json_body: Union[ConvertBotToUserJsonBody, Dict],
        set_system_admin: Optional[bool] = False,
    ) -> StatusOK:
        """Convert a bot into a user

        Convert a bot into a user.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `ConvertBotToUser <https://api.mattermost.com/#operation/ConvertBotToUser>`_
        """

        url = f"/bots/{bot_user_id}/convert_to_user"
        params: Dict[str, Any] = {
            "set_system_admin": set_system_admin,
        }
        params = {k: v for k, v in params.items() if v is not None}

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response



""" Module to access the Emoji endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from ...models import CreateEmojiMultipartData, Emoji, SearchEmojiJsonBody
from ..base import ApiBaseClass


class EmojiApi(ApiBaseClass):
    """Endpoints for creating, getting and interacting with emojis."""

    def get_emoji_list(
        self,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
        sort: Optional[str] = "",
    ) -> Emoji:
        """Get a list of custom emoji

        Get a page of metadata for custom emoji on the system. Since server
        version 4.7, sort using the `sort` query parameter.

        Permissions:
            Must be authenticated.

        Api Reference:
            `GetEmojiList <https://api.mattermost.com/#operation/GetEmojiList>`_
        """

        url = "/emoji"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
            "sort": sort,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Emoji.parse_obj(response.json())

            return response200
        return response

    def create_emoji(
        self,
        *,
        multipart_data: Union[CreateEmojiMultipartData, Dict],
    ) -> Emoji:
        """Create a custom emoji

        Create a custom emoji for the team.

        Permissions:
            Must be authenticated.

        Api Reference:
            `CreateEmoji <https://api.mattermost.com/#operation/CreateEmoji>`_
        """

        url = "/emoji"

        multipart_body_data = CreateEmojiMultipartData.parse_obj(multipart_data)

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Emoji.parse_obj(response.json())

            return response201
        return response

    def get_emoji(
        self,
        emoji_id: str,
    ) -> Emoji:
        """Get a custom emoji

        Get some metadata for a custom emoji.

        Permissions:
            Must be authenticated.

        Api Reference:
            `GetEmoji <https://api.mattermost.com/#operation/GetEmoji>`_
        """

        url = f"/emoji/{emoji_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Emoji.parse_obj(response.json())

            return response200
        return response

    def delete_emoji(
        self,
        emoji_id: str,
    ) -> Emoji:
        """Delete a custom emoji

        Delete a custom emoji.

        Permissions:
            Must have the `manage_team` or `manage_system` permissions
            or be the user who created the emoji.

        Api Reference:
            `DeleteEmoji <https://api.mattermost.com/#operation/DeleteEmoji>`_
        """

        url = f"/emoji/{emoji_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Emoji.parse_obj(response.json())

            return response200
        return response

    def get_emoji_by_name(
        self,
        emoji_name: str,
    ) -> Emoji:
        """Get a custom emoji by name

        Get some metadata for a custom emoji using its name.

        Permissions:
            Must be authenticated.
        Minimum Server Version:
            4.7

        Api Reference:
            `GetEmojiByName <https://api.mattermost.com/#operation/GetEmojiByName>`_
        """

        url = f"/emoji/name/{emoji_name}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Emoji.parse_obj(response.json())

            return response200
        return response

    def get_emoji_image(
        self,
        emoji_id: str,
    ) -> None:
        """Get custom emoji image

        Get the image for a custom emoji.

        Permissions:
            Must be authenticated.

        Api Reference:
            `GetEmojiImage <https://api.mattermost.com/#operation/GetEmojiImage>`_
        """

        url = f"/emoji/{emoji_id}/image"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def search_emoji(
        self,
        *,
        json_body: Union[SearchEmojiJsonBody, Dict],
    ) -> List[Emoji]:
        """Search custom emoji

        Search for custom emoji by name based on search criteria provided in the
        request body. A maximum of 200 results are returned.

        Permissions:
            Must be authenticated.
        Minimum Server Version:
            4.7

        Api Reference:
            `SearchEmoji <https://api.mattermost.com/#operation/SearchEmoji>`_
        """

        url = "/emoji/search"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Emoji.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def autocomplete_emoji(
        self,
        *,
        name: str,
    ) -> Emoji:
        """Autocomplete custom emoji

        Get a list of custom emoji with names starting with or matching the
        provided name. Returns a maximum of 100 results.

        Permissions:
            Must be authenticated.
        Minimum Server Version:
            4.7

        Api Reference:
            `AutocompleteEmoji <https://api.mattermost.com/#operation/AutocompleteEmoji>`_
        """

        url = "/emoji/autocomplete"
        params: Dict[str, Any] = {
            "name": name,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Emoji.parse_obj(response.json())

            return response200
        return response



""" Module to access the Saml endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Dict, Union

from pydantic import BaseModel

from ...models import (
    MigrateAuthToSamlJsonBody,
    ResetSamlAuthDataToEmailJsonBody,
    ResetSamlAuthDataToEmailResponse200,
    SamlCertificateStatus,
    StatusOK,
    UploadSamlIdpCertificateMultipartData,
    UploadSamlPrivateCertificateMultipartData,
    UploadSamlPublicCertificateMultipartData,
)
from ..base import ApiBaseClass


class SamlApi(ApiBaseClass):
    """Endpoints for configuring and interacting with SAML."""

    def migrate_auth_to_saml(
        self,
        *,
        json_body: Union[MigrateAuthToSamlJsonBody, Dict],
    ) -> None:
        """Migrate user accounts authentication type to SAML.

        Migrates accounts from one authentication provider to another. For
        example, you can upgrade your authentication provider from email to
        SAML.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.28

        Api Reference:
            `MigrateAuthToSaml <https://api.mattermost.com/#operation/MigrateAuthToSaml>`_
        """

        url = "/users/migrate_auth/saml"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def get_saml_metadata(
        self,
    ) -> str:
        """Get metadata

        Get SAML metadata from the server. SAML must be configured properly.

        Permissions:
            No permission required.

        Api Reference:
            `GetSamlMetadata <https://api.mattermost.com/#operation/GetSamlMetadata>`_
        """

        url = "/saml/metadata"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = response.json()
            return response200
        return response

    def get_saml_metadata_from_idp(
        self,
    ) -> str:
        """Get metadata from Identity Provider

        Get SAML metadata from the Identity Provider. SAML must be configured
        properly.

        Permissions:
            No permission required.

        Api Reference:
            `GetSamlMetadataFromIdp <https://api.mattermost.com/#operation/GetSamlMetadataFromIdp>`_
        """

        url = "/saml/metadatafromidp"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = response.json()
            return response200
        return response

    def upload_saml_idp_certificate(
        self,
        *,
        multipart_data: Union[UploadSamlIdpCertificateMultipartData, Dict],
    ) -> StatusOK:
        """Upload IDP certificate

        Upload the IDP certificate to be used with your SAML configuration. The
        server will pick a hard-coded filename for the IdpCertificateFile
        setting in your `config.json`.

        Permissions:
            Must have `sysconsole_write_authentication` permission.

        Api Reference:
            `UploadSamlIdpCertificate <https://api.mattermost.com/#operation/UploadSamlIdpCertificate>`_
        """

        url = "/saml/certificate/idp"

        multipart_body_data = UploadSamlIdpCertificateMultipartData.parse_obj(
            multipart_data
        )

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def delete_saml_idp_certificate(
        self,
    ) -> StatusOK:
        """Remove IDP certificate

        Delete the current IDP certificate being used with your SAML
        configuration. This will also disable SAML on your system as this
        certificate is required for SAML.

        Permissions:
            Must have `sysconsole_write_authentication` permission.

        Api Reference:
            `DeleteSamlIdpCertificate <https://api.mattermost.com/#operation/DeleteSamlIdpCertificate>`_
        """

        url = "/saml/certificate/idp"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def upload_saml_public_certificate(
        self,
        *,
        multipart_data: Union[UploadSamlPublicCertificateMultipartData, Dict],
    ) -> StatusOK:
        """Upload public certificate

        Upload the public certificate to be used for encryption with your SAML
        configuration. The server will pick a hard-coded filename for the
        PublicCertificateFile setting in your `config.json`.

        Permissions:
            Must have `sysconsole_write_authentication` permission.

        Api Reference:
            `UploadSamlPublicCertificate <https://api.mattermost.com/#operation/UploadSamlPublicCertificate>`_
        """

        url = "/saml/certificate/public"

        multipart_body_data = UploadSamlPublicCertificateMultipartData.parse_obj(
            multipart_data
        )

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def delete_saml_public_certificate(
        self,
    ) -> StatusOK:
        """Remove public certificate

        Delete the current public certificate being used with your SAML
        configuration. This will also disable encryption for SAML on your system
        as this certificate is required for that.

        Permissions:
            Must have `sysconsole_write_authentication` permission.

        Api Reference:
            `DeleteSamlPublicCertificate <https://api.mattermost.com/#operation/DeleteSamlPublicCertificate>`_
        """

        url = "/saml/certificate/public"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def upload_saml_private_certificate(
        self,
        *,
        multipart_data: Union[UploadSamlPrivateCertificateMultipartData, Dict],
    ) -> StatusOK:
        """Upload private key

        Upload the private key to be used for encryption with your SAML
        configuration. The server will pick a hard-coded filename for the
        PrivateKeyFile setting in your `config.json`.

        Permissions:
            Must have `sysconsole_write_authentication` permission.

        Api Reference:
            `UploadSamlPrivateCertificate <https://api.mattermost.com/#operation/UploadSamlPrivateCertificate>`_
        """

        url = "/saml/certificate/private"

        multipart_body_data = UploadSamlPrivateCertificateMultipartData.parse_obj(
            multipart_data
        )

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def delete_saml_private_certificate(
        self,
    ) -> StatusOK:
        """Remove private key

        Delete the current private key being used with your SAML configuration.
        This will also disable encryption for SAML on your system as this key is
        required for that.

        Permissions:
            Must have `sysconsole_write_authentication` permission.

        Api Reference:
            `DeleteSamlPrivateCertificate <https://api.mattermost.com/#operation/DeleteSamlPrivateCertificate>`_
        """

        url = "/saml/certificate/private"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def get_saml_certificate_status(
        self,
    ) -> SamlCertificateStatus:
        """Get certificate status

        Get the status of the uploaded certificates and keys in use by your SAML
        configuration.

        Permissions:
            Must have `sysconsole_write_authentication` permission.

        Api Reference:
            `GetSamlCertificateStatus <https://api.mattermost.com/#operation/GetSamlCertificateStatus>`_
        """

        url = "/saml/certificate/status"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = SamlCertificateStatus.parse_obj(response.json())

            return response200
        return response

    def reset_saml_auth_data_to_email(
        self,
        *,
        json_body: Union[ResetSamlAuthDataToEmailJsonBody, Dict],
    ) -> ResetSamlAuthDataToEmailResponse200:
        """Reset AuthData to Email

        Reset the AuthData field of SAML users to their email. This is meant to
        be used when the \"id\" attribute is set to an empty value (\"\") from a
        previously non-empty value.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.35

        Api Reference:
            `ResetSamlAuthDataToEmail <https://api.mattermost.com/#operation/ResetSamlAuthDataToEmail>`_
        """

        url = "/saml/reset_auth_data"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = ResetSamlAuthDataToEmailResponse200.parse_obj(response.json())

            return response200
        return response



""" Module to access the Plugins endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from ...models import (
    GetPluginsResponse200,
    InstallMarketplacePluginJsonBody,
    MarketplacePlugin,
    PluginManifest,
    PluginManifestWebapp,
    PluginStatus,
    StatusOK,
    System,
    UploadPluginMultipartData,
)
from ..base import ApiBaseClass


class PluginsApi(ApiBaseClass):
    """Endpoints related to uploading and managing plugins."""

    def get_plugins(
        self,
    ) -> GetPluginsResponse200:
        """Get plugins

        Get a list of inactive and a list of active plugin manifests. Plugins
        must be enabled in the server's config settings.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.4

        Api Reference:
            `GetPlugins <https://api.mattermost.com/#operation/GetPlugins>`_
        """

        url = "/plugins"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = GetPluginsResponse200.parse_obj(response.json())

            return response200
        return response

    def upload_plugin(
        self,
        *,
        multipart_data: Union[UploadPluginMultipartData, Dict],
    ) -> StatusOK:
        """Upload plugin

        Upload a plugin that is contained within a compressed .tar.gz file.
        Plugins and plugin uploads must be enabled in the server's config
        settings.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.4

        Api Reference:
            `UploadPlugin <https://api.mattermost.com/#operation/UploadPlugin>`_
        """

        url = "/plugins"

        multipart_body_data = UploadPluginMultipartData.parse_obj(multipart_data)

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = StatusOK.parse_obj(response.json())

            return response201
        return response

    def install_plugin_from_url(
        self,
        *,
        plugin_download_url: str,
        force: Optional[str] = None,
    ) -> StatusOK:
        """Install plugin from url

        Supply a URL to a plugin compressed in a .tar.gz file. Plugins must be
        enabled in the server's config settings.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.14

        Api Reference:
            `InstallPluginFromUrl <https://api.mattermost.com/#operation/InstallPluginFromUrl>`_
        """

        url = "/plugins/install_from_url"
        params: Dict[str, Any] = {
            "plugin_download_url": plugin_download_url,
            "force": force,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = StatusOK.parse_obj(response.json())

            return response201
        return response

    def remove_plugin(
        self,
        plugin_id: str,
    ) -> StatusOK:
        """Remove plugin

        Remove the plugin with the provided ID from the server. All plugin files
        are deleted. Plugins must be enabled in the server's config settings.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.4

        Api Reference:
            `RemovePlugin <https://api.mattermost.com/#operation/RemovePlugin>`_
        """

        url = f"/plugins/{plugin_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def enable_plugin(
        self,
        plugin_id: str,
    ) -> StatusOK:
        """Enable plugin

        Enable a previously uploaded plugin. Plugins must be enabled in the
        server's config settings.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.4

        Api Reference:
            `EnablePlugin <https://api.mattermost.com/#operation/EnablePlugin>`_
        """

        url = f"/plugins/{plugin_id}/enable"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def disable_plugin(
        self,
        plugin_id: str,
    ) -> StatusOK:
        """Disable plugin

        Disable a previously enabled plugin. Plugins must be enabled in the
        server's config settings.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.4

        Api Reference:
            `DisablePlugin <https://api.mattermost.com/#operation/DisablePlugin>`_
        """

        url = f"/plugins/{plugin_id}/disable"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def get_webapp_plugins(
        self,
    ) -> List[PluginManifestWebapp]:
        """Get webapp plugins

        Get a list of web app plugins installed and activated on the server.

        Permissions:
            No permissions required.
        Minimum Server Version:
            4.4

        Api Reference:
            `GetWebappPlugins <https://api.mattermost.com/#operation/GetWebappPlugins>`_
        """

        url = "/plugins/webapp"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = PluginManifestWebapp.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_plugin_statuses(
        self,
    ) -> List[PluginStatus]:
        """Get plugins status

        Returns the status for plugins installed anywhere in the cluster

        Permissions:
            No permissions required.
        Minimum Server Version:
            4.4

        Api Reference:
            `GetPluginStatuses <https://api.mattermost.com/#operation/GetPluginStatuses>`_
        """

        url = "/plugins/statuses"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = PluginStatus.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_marketplace_plugins(
        self,
        *,
        page: Optional[int] = None,
        per_page: Optional[int] = None,
        filter: Optional[str] = None,
        server_version: Optional[str] = None,
        local_only: Optional[bool] = None,
    ) -> List[MarketplacePlugin]:
        """Gets all the marketplace plugins

        Gets all plugins from the marketplace server, merging data from locally
        installed plugins as well as prepackaged plugins shipped with the
        server.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.16

        Api Reference:
            `GetMarketplacePlugins <https://api.mattermost.com/#operation/GetMarketplacePlugins>`_
        """

        url = "/plugins/marketplace"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
            "filter": filter,
            "server_version": server_version,
            "local_only": local_only,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = MarketplacePlugin.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def install_marketplace_plugin(
        self,
        *,
        json_body: Union[InstallMarketplacePluginJsonBody, Dict],
    ) -> PluginManifest:
        """Installs a marketplace plugin

        Installs a plugin listed in the marketplace server.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.16

        Api Reference:
            `InstallMarketplacePlugin <https://api.mattermost.com/#operation/InstallMarketplacePlugin>`_
        """

        url = "/plugins/marketplace"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = PluginManifest.parse_obj(response.json())

            return response200
        return response

    def get_marketplace_visited_by_admin(
        self,
    ) -> System:
        """Get if the Plugin Marketplace has been visited by at least an admin.

        Retrieves the status that specifies that at least one System Admin has
        visited the in-product Plugin Marketplace.

        Permissions:
            Must have `manage_system` permissions.
        Minimum Server Version:
            5.33

        Api Reference:
            `GetMarketplaceVisitedByAdmin <https://api.mattermost.com/#operation/GetMarketplaceVisitedByAdmin>`_
        """

        url = "/plugins/marketplace/first_admin_visit"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = System.parse_obj(response.json())

            return response200
        return response



""" Module to access the Permissions endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, cast

from ..base import ApiBaseClass


class PermissionsApi(ApiBaseClass):
    """ """

    async def get_ancillary_permissions(
        self,
        *,
        subsection_permissions: Optional[str] = None,
    ) -> List[str]:
        """Return all system console subsection ancillary permissions

        Returns all the ancillary permissions for the corresponding system
        console subsection permissions appended to the requested permission
        subsections.

        Minimum Server Version:
            5.35

        Api Reference:
            `GetAncillaryPermissions <https://api.mattermost.com/#operation/GetAncillaryPermissions>`_
        """

        url = "/permissions/ancillary"
        params: Dict[str, Any] = {
            "subsection_permissions": subsection_permissions,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = cast(List[str], response.json())

            return response200
        return response
